@@ -48,7 +48,7 @@ def _delayed_keyboard_interrupts():
 
 
 class _InstructionContainer:
-    """Holds instructions to modify a table by chaining methods,
+    """Holds instructions to transform a table by chaining methods,
     'map', 'by', 'merge' and 'concat'.
 
     This is a private class. Instead of directly creating an instance of
@@ -91,6 +91,92 @@ class _InstructionContainer:
         })
         return self
 
+    def filter(self, pred):
+        def predicate2gen(pred):
+            def gen(r):
+                if pred(r):
+                    yield r
+            return gen
+
+        self._insts.append({
+            'cmd': 'map',
+            'fn': predicate2gen(pred)
+        })
+        return self
+
+    def update(self, **kwargs):
+        def updatefn(r):
+            for k, v in kwargs.items():
+                r.update({k: v(r)})
+            yield r
+
+        self._insts.append({
+            'cmd': 'map',
+            'fn': updatefn
+        })
+        return self
+
+    def select(self, *cols):
+        def selectfn(r):
+            return {col: r[col] for col in cols}
+
+        self._insts.append({
+            'cmd': 'map',
+            'fn': selectfn
+        })
+        return self
+
+    def join(self, cols, other):
+        def innerjoin_fn(rs1, rs2):
+            if rs1 != [] and rs2 != []:
+                for r1 in rs1:
+                    for r2 in rs2:
+                        r1.update(r2)
+                        yield {col: r1.get(col, None) for col in cols}
+
+        self._insts.append({
+            'cmd': 'merge',
+            'fn': innerjoin_fn,
+            'other': other
+        })
+        return self
+
+    def left_join(self, cols, other):
+        def leftjoin_fn(rs1, rs2):
+            if rs1 != [] and rs2 != []:
+                for r1 in rs1:
+                    for r2 in rs2:
+                        r1.update(r2)
+                        yield {col: r1.get(col, None) for col in cols}
+            elif rs1 != [] and rs2 == []:
+                for r1 in rs1:
+                    yield {col: r1.get(col, None) for col in cols}
+
+        self._insts.append({
+            'cmd': 'merge',
+            'fn': leftjoin_fn,
+            'other': other
+        })
+        return self
+
+    def right_join(self, cols, other):
+        def rightjoin_fn(rs1, rs2):
+            if rs1 != [] and rs2 != []:
+                for r2 in rs2:
+                    for r1 in rs1:
+                        r2.update(r1)
+                        yield {col: r2.get(col, None) for col in cols}
+            elif rs1 == [] and rs2 != []:
+                for r2 in rs2:
+                    yield {col: r2.get(col, None) for col in cols}
+
+        self._insts.append({
+            'cmd': 'merge',
+            'fn': rightjoin_fn,
+            'other': other
+        })
+        return self
+
     def iter(self):
         insts = self._insts
         # If the second instruction is 'by', combine it with the first
