@@ -1,7 +1,7 @@
 import unittest
 import os
 from pathlib import Path
-from itertools import groupby
+from itertools import groupby, product, islice
 from tablemap import Conn
 
 
@@ -117,6 +117,7 @@ class TestBy(unittest.TestCase):
     def setUp(self):
         self.dbfile = 'test/sample.db'
         self.dbfile1 = 'test/sample1.db'
+
     def tearDown(self):
         if Path(self.dbfile).is_file():
             os.remove(self.dbfile)
@@ -129,17 +130,18 @@ class TestBy(unittest.TestCase):
             for r in rs:
                 r['tot'] = tot
                 yield r
+
         def count(rs):
             n = len(rs)
             for r in rs:
                 r['cnt'] = n
                 yield r
- 
+
         conn = Conn(self.dbfile)
         conn['t1'] = table1
         conn['t1_sum'] = conn['t1'].by('col1')\
             .map(sumit).by('tot').map(count)
-        
+
         # on another db
         conn1 = Conn(self.dbfile1)
         conn1['t1_sum'] = conn['t1'].by('col1').map(sumit).by('tot').map(count)
@@ -152,10 +154,10 @@ class TestBy(unittest.TestCase):
     def test_multiple_by(self):
         def foo(r):
             r['col3'] = 1
-            return r 
+            return r
 
         conn = Conn(self.dbfile)
-        conn['t1'] = table1 
+        conn['t1'] = table1
         conn['t1_1'] = conn['t1'].map(foo).by('col1', 'col3')\
             .map(lambda rs: rs)
 
@@ -163,8 +165,89 @@ class TestBy(unittest.TestCase):
         self.assertEqual(conn['t1_1'].list(), conn['t1_2'].list())
 
         conn['t1_1_double'] = conn['t1_1']\
-            .concat(conn['t1'].map(foo).by('col1', 'col3')\
+            .concat(conn['t1'].map(foo).by('col1', 'col3')
                               .map(lambda rs: rs))
         conn['t1_1_double2'] = conn['t1_1'].concat(conn['t1_1'])
 
-        self.assertEqual(conn['t1_1_double'].list(), conn['t1_1_double2'].list())
+        self.assertEqual(conn['t1_1_double'].list(),
+                         conn['t1_1_double2'].list())
+
+    def test_itertools(self):
+
+        conn = Conn(self.dbfile)
+        conn['t1'] = table1
+        conn['t2'] = table2
+
+        conn['t1_1'] = islice(conn['t1'].iter(), 0, 3)
+
+        self.assertEqual(len(conn['t1_1'].list()), 3)
+
+        xs = conn['t1'].iter()
+        ys = conn['t2'].iter()
+
+        conn['t1_2'] = ({**x, **y} for x, y in product(xs, ys))
+
+        n1 = len(conn['t1'].list())
+        n2 = len(conn['t2'].list())
+        n3 = len(conn['t1_2'].list())
+
+        self.assertEqual(n3, n1 * n2)
+
+
+class TestDerivs(unittest.TestCase):
+    def setUp(self):
+        self.dbfile = 'test/sample.db'
+
+    def tearDown(self):
+        if Path(self.dbfile).is_file():
+            os.remove(self.dbfile)
+
+    def test_filter(self):
+        conn = Conn(self.dbfile)
+
+        conn['t1'] = table1
+        conn['t1_1'] = conn['t1'].filter(lambda r: r['col2'] > 2)
+
+        self.assertEqual(len(conn['t1_1'].list()), 5)
+
+    def test_update(self):
+        conn = Conn(self.dbfile)
+        conn['t1'] = table1
+        conn['t1_1'] = conn['t1'].update(
+            col2=lambda r: r['col2'] + 1,
+            col3=lambda r: r['col1'] + str(r['col2'])
+        )
+        # table is fetched as it's inserted. not sure if it's a feature.
+        self.assertEqual(conn['t1_1'].list()[0],
+                         {'col1': 'a', 'col2': 5, 'col3': 'a5'})
+
+    def test_select(self):
+        conn = Conn(self.dbfile)
+        conn['t1'] = table1
+        conn['t1_1'] = conn['t1'].update(
+            col2=lambda r: r['col2'] + 1,
+            col3=lambda r: r['col1'] + str(r['col2'])
+        )
+
+        r = next(conn['t1_1'].select('col1', 'col3').iter())
+        self.assertEqual(len(r), 2)
+
+    def test_join(self):
+        conn = Conn(self.dbfile)
+        conn['t1'] = table1
+        conn['t2'] = table2
+
+        conn['t1_1'] = conn['t1'].by('col1').join(['col1', 'col2', 'col3'],
+                                                  conn['t2'].by('col1'))
+
+        self.assertEqual(len(conn['t1_1'].list()), 3)
+
+        conn['t1_2'] = conn['t1'].by('col1').left_join(['col1', 'col2', 'col3'],
+                                                       conn['t2'].by('col1'))
+
+        self.assertEqual(len(conn['t1_2'].list()), 8)
+
+        conn['t1_3'] = conn['t1'].by('col1').right_join(['col1', 'col2', 'col3'],
+                                                        conn['t2'].by('col1'))
+
+        self.assertEqual(len(conn['t1_3'].list()), 5)
